

var CircleDiagram = null;

(function($){
  
  CircleDiagram = Class.extend({
      init: function(options) {
	 var self = this;	    
	 self.defaults = {
	     canvasId: 'myCanvas',
	     radius: 45,
	     width: 20,
	     color: '#d6d6cc',
	     start: 0,
	     end: 3,
	     x: 100,
	     y: 100,
	     shadow: true,
	     data: [],
	     
	     proz: null,
	     bigLabel: false
	 };
	 self.oneProz = 0.02;
	 self.startPos = 1.5;
	 self.currPos = null;
	 self.endPos = 0;
	 
	 self.animationIsStop = false;	 
	 self.animationPoint = self.startPos;
	 self.animationStopPoint = 0;
	 self.animationLastStopItemI = null;
	 
	 self.opt = $.extend(self.defaults, options);
	 self.opt.canvas = document.getElementById(self.opt.canvasId);
	 self.items = Array();
	 
	 self.img = new Image();
	 self.img.src="./img/pf.png";	 
	 
	 self.draw(self.opt.start,self.opt.end);
	 self.createItems();
      },
      
      
      draw: function(start, end) {
	  console.log('erst');
	    var self = this;
	    self.endPos = end;
	    self.currPos = start;
	  
	    var canvas = self.opt.canvas;
	    var context = canvas.getContext('2d');
	    var x = self.opt.x;
	    var y = self.opt.y;
	    var radius = self.opt.radius;
	    var startAngle = start * Math.PI;
	    var endAngle = end * Math.PI;
	    var counterClockwise = false;

	    if(self.opt.shadow == true) {
		context.shadowOffsetX = 2;
		context.shadowOffsetY = 2;
		context.shadowBlur = 5;
		context.shadowColor = "#2f2f2f";
	    }else {
		context.shadowOffsetX = 0;
		context.shadowOffsetY = 0;
		context.shadowBlur = 0;
	    }

	    context.beginPath();
	    context.arc(x, y, radius, startAngle, endAngle, counterClockwise);
	    context.lineWidth = self.opt.width;

	    // line color
	    context.strokeStyle = self.opt.color;
	    context.stroke();
	    
	    context.beginPath();
	    context.shadowOffsetX = 0;
	    context.shadowOffsetY = 0;
	    context.shadowBlur = 0;    
	    
	    
	    if(self.opt.proz) {
		context.lineWidth = 1;	    
		var w = ( (end - (end-start)/2) * Math.PI);
		var _x_s = (Math.cos(w)*(radius-(self.opt.width/2) ));
		var _y_s = (Math.sin(w)*(radius-(self.opt.width/2) ));
		var _x = (Math.cos(w)*(radius+30));
		var _y = (Math.sin(w)*(radius+30));

		var _x_t = (Math.cos(w)*(radius+(self.opt.width*1.5) ));
		var _y_t = (Math.sin(w)*(radius+(self.opt.width*1.5) ));
		_x_t = _x_t -10;
		context.moveTo( x+_x_s , y+_y_s );
		
		if(self.opt.bigLabel) {	
		    context.lineTo(  x+_x , y+_y );
		    _x_t = _x_t +13;
		    _y_t = _y_t -10;
		    context.fillStyle = self.opt.color;
		    self.wrapText(context, "Das war deine Auswahl.", x+_x_t, y+_y_t+13, 120, 13);
		}else {
		    context.fillStyle = '#ffffff'; 
		}
		context.stroke();

		context.beginPath();
		context.font = '10pt Calibri';
		context.fillText(self.opt.proz+"%", x+_x_t , y+_y_t, 130, 20);
		
		if(self.opt.bigLabel) {	
		    context.save(); 
		    context.translate(x, y);		    
		    context.rotate( (3.6*(self.opt.proz/2) )  *Math.PI/180);
		    context.drawImage(self.img,-(self.img.width/2), -(self.img.height/2));
		    context.restore(); 
		}
		context.stroke();
	    }
      },
      
      updateDraw: function(v) {
	  var self = this;	  
	  self.currPos = self.currPos + v;
	  self.endPos = self.endPos + v;
	  self.draw(self.currPos, self.endPos);
      },
      
      createItems: function() {
	  var self = this;
	  self.currPos = self.startPos;	
	
	  var r=0;
	  var end = null;	  
	  for(var i=self.opt.data.length-1; i > -1; i--) {
	      end = self.currPos + (self.opt.data[i].proz * self.oneProz);  
	      //console.log(self.currPos+' '+end+' | '+self.opt.data[i].proz);
	      var item = new CircleDiagram({
		  canvasId: self.opt.canvasId,
		  radius: ((i==0)? (self.opt.radius+5):self.opt.radius),
		  width: 20,
		  start: self.currPos,
		  end: end,
		  color: self.opt.data[i].color,
		  x: self.opt.x,
		  y: self.opt.y,
		  shadow: ((i==0)? true:false),
		  proz: self.opt.data[i].proz,
		  bigLabel: ((i==0)? true:false)
	      }); 
	      r+=5;
	      self.items.push(item);
	  }
	  
	  if(self.opt.data.length > 0) {
	      self.currPos =  self.items[self.items.length-1].opt.end;
	      self.animate();
	  }
      },
      
      animate: function() {
	  var self = this;	  
	  if(self.animateItems()) {	      
	      reqAnimFrame = window.mozRequestAnimationFrame ||
		  window.webkitRequestAnimationFrame || 
		  window.msRequestAnimationFrame     ||
		  window.oRequestAnimationFrame;

	      reqAnimFrame(function() {
		self.animate();
	      });
	  }
	},
	
	animateItems: function() {   
	    var self = this;

	    var canvas = self.opt.canvas;
	    var context = canvas.getContext('2d');
	    context.clearRect(0, 0, canvas.width, canvas.height);

	    self.draw(self.opt.start,self.opt.end);	
	    
	    if(self.animationLastStopItemI==null) {
		self.items[self.items.length-1].animationIsStop = true;
		self.animationStopPoint = self.items[self.items.length-1].opt.end-0.01;
		self.animationLastStopItemI = self.items.length-1;
	    }

	    var update = false;
	    self.animationPoint+=0.01;
	    for(var i=0; i < self.items.length; i++) {		
	      if(self.animationStopPoint < self.items[i].currPos && i == self.animationLastStopItemI-1 && self.items[i].animationIsStop == false) {
		  self.items[i].animationIsStop = true;
		  self.animationStopPoint = self.items[i].endPos-0.01;
		  self.animationLastStopItemI = i;
		  self.items[i].updateDraw(0);
	      }else if(self.items[i].animationIsStop == false) {
		  self.items[i].updateDraw(0.01);
		  update = true;
	      }else {
		  self.items[i].updateDraw(0);
	      }
	    }
	    return update;
	},
	
	
	
	wrapText: function(context, text, x, y, maxWidth, lineHeight) {
	    var words = text.split(' ');
	    var line = '';
	    
	    for(var n = 0; n < words.length; n++) {
		var testLine = line + words[n] + ' ';
		var metrics = context.measureText(testLine);
		var testWidth = metrics.width;
		if(testWidth > maxWidth) {
		    context.fillText(line, x, y);
		    line = words[n] + ' ';
		    y += lineHeight;
		}
		else {
		    line = testLine;
		}
	    }
	    context.fillText(line, x, y);
	}
	
      
  });
  
})(jQuery);